@@ -12,10 +12,9 @@ from Products.CMFPlone.tests import PloneTestCase
 from Acquisition import aq_base
 from Globals import REPLACEABLE
 from DateTime import DateTime
-from Products.CMFCore.CMFCorePermissions import AccessInactivePortalContent
+from Products.CMFCore.permissions import AccessInactivePortalContent
 from Products.CMFPlone import transaction
 
-from Products.CMFPlone.CatalogTool import ExtensibleIndexableObjectRegistry
 from Products.CMFPlone.CatalogTool import ExtensibleIndexableObjectWrapper
 from Products.CMFPlone.CatalogTool import _eioRegistry
 
@@ -30,57 +29,32 @@ group2 = 'g2'
 
 base_content = ['Members', 'events', 'news', 'previous', default_user, 'doc']
 
-try:
-    import Products.TextIndexNG2
-    txng_version = 2
-except:
-    txng_version = 0
-
 
 class TestCatalogSetup(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
         self.catalog = self.portal.portal_catalog
 
-    if txng_version == 2:
+    def testSearchableTextIsZCTextIndex(self):
+        # SearchableText index should be a ZCTextIndex
+        itype = self.catalog.Indexes['SearchableText'].__class__.__name__
+        self.assertEqual(itype, 'ZCTextIndex')
 
-        def testSearchableTextIsTextIndexNG(self):
-            # SearchableText index should be a TextIndexNG
-            itype = self.catalog.Indexes['SearchableText'].__class__.__name__
-            self.assertEqual(itype, 'TextIndexNG')
+    def testDescriptionIsZCTextIndex(self):
+        # Description index should be a ZCTextIndex
+        itype = self.catalog.Indexes['Description'].__class__.__name__
+        self.assertEqual(itype, 'ZCTextIndex')
 
-        def testDescriptionIsTextIndexNG(self):
-            # Description index should be a TextIndexNG
-            itype = self.catalog.Indexes['Description'].__class__.__name__
-            self.assertEqual(itype, 'TextIndexNG')
+    def testTitleIsZCTextIndex(self):
+        # Title index should be a ZCTextIndex
+        itype = self.catalog.Indexes['Title'].__class__.__name__
+        self.assertEqual(itype, 'ZCTextIndex')
 
-        def testTitleIsTextIndexNG(self):
-            # Title index should be a TextIndexNG
-            itype = self.catalog.Indexes['Title'].__class__.__name__
-            self.assertEqual(itype, 'TextIndexNG')
-
-    else:
-
-        def testSearchableTextIsZCTextIndex(self):
-            # SearchableText index should be a ZCTextIndex
-            itype = self.catalog.Indexes['SearchableText'].__class__.__name__
-            self.assertEqual(itype, 'ZCTextIndex')
-
-        def testDescriptionIsZCTextIndex(self):
-            # Description index should be a ZCTextIndex
-            itype = self.catalog.Indexes['Description'].__class__.__name__
-            self.assertEqual(itype, 'ZCTextIndex')
-
-        def testTitleIsZCTextIndex(self):
-            # Title index should be a ZCTextIndex
-            itype = self.catalog.Indexes['Title'].__class__.__name__
-            self.assertEqual(itype, 'ZCTextIndex')
-
-        def testPloneLexiconIsZCTextLexicon(self):
-            # Lexicon should be a ZCTextIndex lexicon
-            self.failUnless(hasattr(aq_base(self.catalog), 'plone_lexicon'))
-            self.assertEqual(self.catalog.plone_lexicon.meta_type,\
-                             'ZCTextIndex Lexicon')
+    def testPloneLexiconIsZCTextLexicon(self):
+        # Lexicon should be a ZCTextIndex lexicon
+        self.failUnless(hasattr(aq_base(self.catalog), 'plone_lexicon'))
+        self.assertEqual(self.catalog.plone_lexicon.meta_type,\
+                         'ZCTextIndex Lexicon')
 
     def testPathIsExtendedPathIndex(self):
         # path index should be an ExtendedPathIndex
@@ -176,6 +150,14 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
         self.folder.invokeFactory('Document', id='doc', title='Foo', description='Bar')
         self.catalog.unindexObject(self.folder.doc)
 
+    def assertResults(self, result, expect):
+        # Verifies ids of catalog results against expected ids
+        lhs = [r.getId for r in result]
+        lhs.sort()
+        rhs = list(expect)
+        rhs.sort()
+        self.assertEqual(lhs, rhs)
+
     def testFixture(self):
         self.assertEqual(self.folder.doc.getId(), 'doc')
         self.assertEqual(self.folder.doc.Title(), 'Foo')
@@ -274,7 +256,7 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
     def testIndexTitleOnly(self):
         # Indexing should only index the Title
         #
-        # XXX: This does not work as expected. The object
+        # TODO: This does not work as expected. The object
         # appears to be in the catalog but is not returned
         # by searchResults()!?
         #
@@ -290,7 +272,7 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
     def testIndexIdOnly(self):
         # Indexing should only index the id
         #
-        # XXX: Demonstrate that the behavior is independent
+        # TODO: Demonstrate that the behavior is independent
         # of index type.
         #
         self.catalog.indexObject(self.folder.doc, idxs=['id'])
@@ -302,6 +284,38 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
         self.assertEqual(len(self.catalog(Title='Foo')), 0)
         self.assertEqual(len(self.catalog(Description='Bar')), 0)
 
+    def testClearFindAndRebuildRemovesBadContent(self):
+        # Index the doc for consistency
+        self.catalog.indexObject(self.folder.doc)
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content)
+        # index an object which shouldn't be there
+        self.catalog.indexObject(self.portal.portal_skins)
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content+['portal_skins'])
+        self.catalog.clearFindAndRebuild()
+        # This will remove the extraneous item and add the document added
+        # in afterSetup
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content)
+
+    def testClearFindAndRebuildAddsMissingContent(self):
+        # Index the doc for consistency
+        self.catalog.indexObject(self.folder.doc)
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content)
+        # index an object which shouldn't be there
+        self.catalog.unindexObject(self.portal.Members)
+        altered_content = base_content[:]
+        altered_content.remove('Members')
+        res = self.catalog.searchResults()
+        self.assertResults(res, altered_content)
+        self.catalog.clearFindAndRebuild()
+        # This will add the missing item and also the document added
+        # in afterSetup
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content)
+
 
 class TestCatalogSearching(PloneTestCase.PloneTestCase):
 
@@ -310,7 +324,7 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         self.workflow = self.portal.portal_workflow
         self.groups = self.portal.portal_groups
 
-        self.portal.acl_users._doAddUser(user2, 'secret', [], [], [])
+        self.portal.acl_users._doAddUser(user2, 'secret', [], [])
 
         self.folder.invokeFactory('Document', id='doc', text='foo')
         self.folder.invokeFactory('Folder', id='folder2')
@@ -327,7 +341,9 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         self.groups.groupWorkspacesCreationFlag = 0
         self.groups.addGroup(group2, None, [], [])
         group = self.groups.getGroupById(group2)
+        self.loginPortalOwner() # GRUF 3.52
         group.addMember(user2)
+        self.login(default_user) # Back to normal
         prefix = self.portal.acl_users.getGroupPrefix()
         return '%s%s' % (prefix, group2)
 
@@ -340,7 +356,7 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
 
     def testSearchReturnsDocument(self):
         # Document should be found when owner does a search
-        self.assertEqual(self.catalog(SearchableText='foo')[0].id, 'doc')
+        self.assertEqual(self.catalog(SearchableText='aaa')[0].id, 'aaa')
 
     def testSearchDoesNotReturnDocument(self):
         # Document should not be found when user2 does a search
@@ -358,7 +374,7 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         self.assertEqual(len(self.catalog(SearchableText='aaa AND ccc')), 1)
 
     def testSearchReturnsDocumentUsing_OR(self):
-        # Two documents (aaa, bbb)  should be found when owner does a search using OR 
+        # Two documents (aaa, bbb)  should be found when owner does a search using OR
         results = self.catalog(SearchableText='aaa OR bbb')
         self.assertEqual(len(results), 2)
 
@@ -368,7 +384,7 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         groupname = self.addUser2ToGroup()
         self.folder.folder_localrole_edit('add', [groupname], 'Owner')
         self.login(user2)
-        self.assertEqual(self.catalog(SearchableText='foo')[0].id, 'doc')
+        self.assertEqual(self.catalog(SearchableText='aaa')[0].id, 'aaa')
 
     def testSearchRespectsLocalRoleAcquisition(self):
         # After adding a group with access rights and containing user2,
@@ -377,7 +393,7 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         self.folder.folder_localrole_edit('add', [groupname], 'Owner')
         self.login(user2)
         # Local Role works in subfolder
-        self.assertEqual(self.catalog(SearchableText='bar')[0].id, 'doc2')
+        self.assertEqual(self.catalog(SearchableText='bbb')[0].id, 'bbb')
 
     def testSearchRespectsLocalRoleAcquisitionDisabled(self):
         # After adding a group with access rights and containing user2,
@@ -386,9 +402,8 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         groupname = self.addUser2ToGroup()
         self.folder.folder_localrole_edit('add', [groupname], 'Owner')
         # Acquisition off for folder2
-        self.login(default_user)
         self.folder.folder2.folder_localrole_set(use_acquisition=0)
-        #Everything in subfolder should be invisible
+        # Everything in subfolder should be invisible
         self.login(user2)
         self.failIf(self.catalog(SearchableText='bar'))
 
@@ -456,8 +471,8 @@ class TestFolderCataloging(PloneTestCase.PloneTestCase):
         self.folder.invokeFactory('Folder', id='foo')
 
     def testFolderTitleIsUpdatedOnEdit(self):
-        # Test for catalog that searches to ensure folder titles are 
-        # updated in the catalog. 
+        # Test for catalog that searches to ensure folder titles are
+        # updated in the catalog.
         title = 'Test Folder - Snooze!'
         self.folder.foo.folder_edit(title, '')
         results = self.catalog(Title='Snooze')
@@ -467,10 +482,10 @@ class TestFolderCataloging(PloneTestCase.PloneTestCase):
             self.assertEqual(result.id, 'foo')
 
     def testFolderTitleIsUpdatedOnRename(self):
-        # Test for catalog that searches to ensure folder titles are 
-        # updated in the catalog. 
+        # Test for catalog that searches to ensure folder titles are
+        # updated in the catalog.
         title = 'Test Folder - Snooze!'
-        transaction.commit(1) # make rename work
+        transaction.savepoint(optimistic=True) # make rename work
         self.folder.foo.folder_edit(title, '', id='bar')
         results = self.catalog(Title='Snooze')
         self.failUnless(results)
@@ -492,7 +507,7 @@ class TestFolderCataloging(PloneTestCase.PloneTestCase):
     def testFolderTitleIsUpdatedOnFolderRename(self):
         # The bug in fact talks about folder_rename
         title = 'Test Folder - Snooze!'
-        transaction.commit(1) # make rename work
+        transaction.savepoint(optimistic=True) # make rename work
         foo_path = '/'.join(self.folder.foo.getPhysicalPath())
         self.folder.folder_rename(paths=[foo_path], new_ids=['bar'], new_titles=[title])
         results = self.catalog(Title='Snooze')
@@ -579,7 +594,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         # that blames permissions or lack of support by the obj. The obj is a
         # Plone Document, and the owner of doc2 is portal_owner. Harumph.
 
-        transaction.commit(1)
+        transaction.savepoint(optimistic=True)
 
         self.folder.manage_renameObjects(['doc2'], ['buzz'])
         folder_docs = self.catalog(portal_type = 'Document',
@@ -621,7 +636,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         all_objs = self.catalog()
         sorted_objs = self.catalog(sort_on='getObjPositionInParent')
         self.failUnlessEqual(len(all_objs), len(sorted_objs))
-        
+
         members = self.portal.Members
         members_path = '/'.join(members.getPhysicalPath())
         members_query = self.catalog(path=members_path)
@@ -653,6 +668,18 @@ class TestCatalogBugs(PloneTestCase.PloneTestCase):
         self.folder.manage_pasteObjects(cb)
         self.failUnless(hasattr(aq_base(self.folder), 'portal_catalog'))
 
+    def testPastingCatalogPreservesTextIndexes(self):
+        # Pasting the catalog should not cause indexes to be removed.
+        self.setRoles(['Manager'])
+        self.catalog.__replaceable__ = REPLACEABLE
+        cb = self.portal.manage_copyObjects(['portal_catalog'])
+        self.folder.manage_pasteObjects(cb)
+        self.failUnless(hasattr(aq_base(self.folder), 'portal_catalog'))
+        cat = self.folder.portal_catalog
+        self.failUnless('SearchableText' in cat.indexes())
+        # CMF added lexicons should stick around too
+        self.failUnless(hasattr(aq_base(cat), 'plaintext_lexicon'))
+
     def testCanRenamePortalIfLexiconExists(self):
         # Should be able to rename a Plone portal
         # This test is to demonstrate that http://plone.org/collector/1745
@@ -849,7 +876,7 @@ class TestExtensibleIndexableObjectWrapper(PloneTestCase.PloneTestCase):
         self.failIf(is_folderish(i))
 
     def test_is_folderishWithFolder(self):
-        f = dummy.ATFolder('struct_folder')
+        f = dummy.Folder('struct_folder')
         self.failUnless(is_folderish(f))
 
     def test_is_folderishWithNonStructuralFolder(self):
