@@ -7,14 +7,14 @@ if __name__ == '__main__':
     execfile(os.path.join(sys.path[0], 'framework.py'))
 
 from Testing import ZopeTestCase
-from Products.CMFCore import CMFCorePermissions
+from Products.CMFCore.permissions import AccessInactivePortalContent
 from Products.CMFPlone.tests import PloneTestCase
 from Products.CMFPlone.tests import dummy
 
 from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
-from OFS.SimpleItem import SimpleItem
 from Acquisition import aq_base
 from DateTime import DateTime
+from tempfile import mkstemp
 
 
 class TestPortalCreation(PloneTestCase.PloneTestCase):
@@ -33,6 +33,8 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
         self.factory = self.portal.portal_factory
         self.cc = self.portal.cookie_authentication
         self.skins = self.portal.portal_skins
+        self.transforms = self.portal.portal_transforms
+        self.javascripts = self.portal.portal_javascripts
 
     def testPloneSkins(self):
         # Plone skins should have been set up
@@ -57,7 +59,7 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
         self.failUnless(self.cp.__dict__.has_key('groups'))
 
     def testWorkflowIsActionProvider(self):
-        # XXX: This change has been backed out and the test inverted!
+        # This change has been backed out and the test inverted!
         # Remove portal_workflow by default.  We are falling back to
         # our use of the 'review_slot'.  There are no places using
         # the worklist ui anymore directly from the listFilteredActionsFor
@@ -66,7 +68,7 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
 
     def testReplyTabIsOff(self):
         # Ensure 'reply' tab is turned off
-        # XXX NOTE: ActionProviderBAse should have a 'getActionById'
+        # NOTE: ActionProviderBAse should have a 'getActionById'
         # that does this for x in: if x == id
         dt_actions = self.portal.portal_discussion.listActions()
         reply_visible=1
@@ -83,7 +85,7 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
         self.failIf('plone_workflow' in lpf_chain)
 
     def testMembersFolderMetaType(self):
-        # Members folder should have meta_type 'Large Plone Folder'
+        # Members folder should have meta_type 'ATBTreeFolder'
         members = self.membership.getMembersFolder()
         #self.assertEqual(members.meta_type, 'Large Plone Folder')
         self.assertEqual(members.meta_type, 'ATBTreeFolder')
@@ -140,8 +142,8 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
 
     def testFolderHasFolderListingAction(self):
         # Folders should have a 'folderlisting' action
-        topic = self.types.getTypeInfo('Folder')
-        for action in topic._cloneActions():
+        folder = self.types.getTypeInfo('Folder')
+        for action in folder._cloneActions():
             if action.id == 'folderlisting':
                 break
         else:
@@ -257,13 +259,21 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
                 self.fail("Actions tool still has 'News' action")
 
     def testNewsFolderIsIndexed(self):
-        # News folder should be cataloged
+        # News (smart) folder should be cataloged
         res = self.catalog(id='news')
         self.assertEqual(len(res), 1)
         self.assertEqual(res[0].getId, 'news')
         self.assertEqual(res[0].Title, 'News')
         self.assertEqual(res[0].Description, 'Site News')
 
+    def testEventsFolderIsIndexed(self):
+        # Events (smart) folder should be cataloged
+        res = self.catalog(id='events')
+        self.assertEqual(len(res), 1)
+        self.assertEqual(res[0].getId, 'events')
+        self.assertEqual(res[0].Title, 'Events')
+        self.assertEqual(res[0].Description, 'Site Events')
+
     def testNewsTopic(self):
         # News topic is in place as default view and has a criterion to show
         # only News Items, and uses the folder_summary_view.
@@ -473,10 +483,10 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
 
     def testOwnerHasAccessInactivePermission(self):
         permission_on_role = [p for p in self.portal.permissionsOfRole('Owner')
-            if p['name'] == CMFCorePermissions.AccessInactivePortalContent][0]
+            if p['name'] == AccessInactivePortalContent][0]
         self.failUnless(permission_on_role['selected'])
         cur_perms = self.portal.permission_settings(
-                            CMFCorePermissions.AccessInactivePortalContent)[0]
+                            AccessInactivePortalContent)[0]
         self.failUnless(cur_perms['acquire'])
 
     def testSyndicationEnabledByDefault(self):
@@ -513,7 +523,7 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
         self.folder.invokeFactory('Document','index_html')
         acts = self.actions.listFilteredActionsFor(self.folder.index_html)
         buttons = acts['object_buttons']
-        self.assertEqual(len(buttons), 3)
+        self.assertEqual(len(buttons), 4)
         urls = [a['url'] for a in buttons]
         for url in urls:
             self.failIf('index_html' in url, 'Action wrongly applied to default page object %s'%url)
@@ -524,7 +534,7 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
         self.folder.invokeFactory('Document','index_html')
         acts = self.actions.listFilteredActionsFor(self.folder.index_html)
         buttons = acts['object_buttons']
-        self.assertEqual(len(buttons), 3)
+        self.assertEqual(len(buttons), 4)
         urls = [(a['id'],a['url']) for a in buttons]
         for url in urls:
             # ensure that e.g. the 'copy' url contains object_copy
@@ -537,9 +547,9 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
         self.folder.cb_dataValid = True
         acts = self.actions.listFilteredActionsFor(self.folder)
         buttons = acts['object_buttons']
-        self.assertEqual(len(buttons),4)
+        self.assertEqual(len(buttons),5)
         ids = [(a['id']) for a in buttons]
-        self.assertEqual(ids, ['cut','copy','paste','delete'])
+        self.assertEqual(ids, ['cut','copy','paste','delete', 'rename'])
 
     def testPortalSharingActionIsLocalRoles(self):
         fti = getattr(self.types, 'Plone Site')
@@ -592,12 +602,72 @@ class TestPortalCreation(PloneTestCase.PloneTestCase):
                                         if p['name'] == 'View Groups'][0]
         self.failUnless(member_has_permission['selected'])
 
+    def testDiscussionItemHasNoWorkflow(self):
+        self.assertEqual(self.workflow.getChainForPortalType('Discussion Item'), ())
+
+    def testFolderHasFolderListingView(self):
+        # Folder type should allow 'folder_listing'
+        self.failUnless('folder_listing' in self.types.Folder.view_methods)
+
+    def testFolderHasSummaryView(self):
+        # Folder type should allow 'folder_summary_view'
+        self.failUnless('folder_summary_view' in self.types.Folder.view_methods)
+
+    def testFolderHasTabularView(self):
+        # Folder type should allow 'folder_tabular_view'
+        self.failUnless('folder_tabular_view' in self.types.Folder.view_methods)
+
+    def testFolderHasAlbumView(self):
+        # Folder type should allow 'atct_album_view'
+        self.failUnless('atct_album_view' in self.types.Folder.view_methods)
+
+    def testConfigurableSafeHtmlTransform(self):
+        # The safe_html transformation should be configurable
+        try:
+            self.transforms.safe_html.get_parameter_value('disable_transform')
+        except (AttrbuteError, KeyError):
+            self.fail('safe_html transformation not updated')
+
+    def testNavtreePropertiesNormalized(self):
+        ntp = self.portal.portal_properties.navtree_properties
+        toRemove = ['skipIndex_html', 'showMyUserFolderOnly', 'showFolderishSiblingsOnly',
+                    'showFolderishChildrenOnly', 'showNonFolderishObject', 'showTopicResults',
+                    'rolesSeeContentView', 'rolesSeeUnpublishedContent', 'rolesSeeContentsView ',
+                    'batchSize', 'sortCriteria', 'croppingLength', 'forceParentsInBatch', 
+                    'rolesSeeHiddenContent', 'typesLinkToFolderContents']
+        toAdd = {'name' : '', 'root' : '/', 'currentFolderOnlyInNavtree' : False}
+        for property in toRemove:
+            self.assertEqual(ntp.getProperty(property, None), None)
+        for property, value in toAdd.items():
+            self.assertEqual(ntp.getProperty(property), value)
+        self.assertEqual(ntp.getProperty('bottomLevel'), 0)
+
+    def testvcXMLRPCRemoved(self):
+        # vcXMLRPC.js should no longer be registered
+        self.failIf('vcXMLRPC.js' in self.javascripts.getResourceIds())
+
+    def testActionDropDownMenuIcons(self):
+        # Object buttons should have icons
+        icons = self.icons.listActionIcons()
+        def assertIcon(action_id):
+            for icon in icons:
+                if (icon.getActionId() == action_id and
+                    icon.getCategory() == 'object_buttons'):
+                    break
+            else:
+                self.fail("Action icons tool has no '%s' icon" % action_id)
+        assertIcon('cut')
+        assertIcon('copy')
+        assertIcon('paste')
+        assertIcon('delete')
+
 
 class TestPortalBugs(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
         self.membership = self.portal.portal_membership
         self.members = self.membership.getMembersFolder()
+        self.catalog = self.portal.portal_catalog
         # Fake the Members folder contents
         self.members._setObject('index_html', ZopePageTemplate('index_html'))
 
@@ -635,6 +705,28 @@ class TestPortalBugs(PloneTestCase.PloneTestCase):
         self.app._delObject(PloneTestCase.portal_name)
         self.failUnless(self.foo.before_delete_called())
 
+    def testExportImportLosesTextIndexes(self):
+        # Importing a portal .zexp loses text indexes? (#4803)
+        self.loginPortalOwner()
+        tempfile = mkstemp('.zexp')
+        tempname = tempfile[1]
+        try:
+            # Export the portal
+            self.portal._p_jar.exportFile(self.portal._p_oid, tempname)
+            # Nuke it
+            self.app._delObject(PloneTestCase.portal_name)
+            # Import the portal
+            self.app._importObjectFromFile(tempname, set_owner=0)
+            # Now check the indexes are still present
+            for index in ('Description', 'Title', 'SearchableText'):
+                try:
+                    self.catalog.Indexes[index]
+                except KeyError:
+                    self.fail('Index %s missing after export/import!' % index)
+        finally:
+            os.close(tempfile[0])
+            os.remove(tempname)
+
 
 class TestManagementPageCharset(PloneTestCase.PloneTestCase):
 
